package minerva

import (
	"encoding/json"
	"os"
)

type PersistentEngine interface {
	Write(interface{})error
	Read()(Minervas, error)
}

type PersistentFile struct {
	dest string
}

func NewPersistentFile(path string)*PersistentFile{
	return &PersistentFile{
		dest: path,
	}
}

func (pf *PersistentFile)Write(v interface{})error{
	data, err := json.MarshalIndent(v, "", "\t")
	if err != nil {
		return err
	}
	file, err := os.OpenFile(pf.dest, os.O_CREATE|os.O_WRONLY, os.ModePerm)
	if err != nil {
		return err
	}
	defer file.Close()
	_, err = file.Write(data)
	return err
}

func (pf *PersistentFile)Read()(Minervas, error){
	pf.checkDest()
	data, err := os.ReadFile(pf.dest)
	if err != nil && !os.IsNotExist(err) {
		return nil, err
	}
	m := make(Minervas)
	if err == nil && len(data) > 1 {
		err = json.Unmarshal(data, &m)
	}
	return m, err
}

func (pf *PersistentFile)checkDest()error{
	_, err := os.Stat(pf.dest)
	if os.IsExist(err){
		return nil
	}
	f, err := os.Create(pf.dest)
	if err != nil {
		return err
	}
	defer f.Close()
	return nil
}